@@ -69,6 +69,7 @@ def yaml_path(path):
 class BootLoader(Enum):
     uboot = 'u-boot'
     grub = 'grub'
+    lk = 'lk'
 
 
 @yaml_path('volumes:<volume name>:schema')
@@ -89,6 +90,10 @@ class StructureRole(Enum):
     mbr = 'mbr'
     system_boot = 'system-boot'
     system_data = 'system-data'
+    system_seed = 'system-seed'
+    system_save = 'system-save'
+    system_boot_select = 'system-boot-select"'
+    system_boot_image = 'system-boot-image'
 
 
 class Enumify:
@@ -188,7 +193,7 @@ GadgetYAML = Schema({
     Optional('device-tree'): str,
     Optional('format'): YAMLFormat,
     Required('volumes'): {
-        Match('^[-a-zA-Z0-9]+$'): Schema({
+        Match('^[a-zA-Z0-9][-a-zA-Z0-9]*$'): Schema({
             Optional('schema', default='gpt' if has_new_voluptuous()
                      else VolumeSchema.gpt):
                 Enumify(VolumeSchema),
@@ -288,6 +293,8 @@ class GadgetSpec:
     volumes = attr.ib()
     defaults = attr.ib()
     format = attr.ib()
+    # Additional u-i internal metadata, not part of the spec
+    seeded = attr.ib()
 
 
 def parse(stream_or_string):
@@ -337,6 +344,10 @@ def parse(stream_or_string):
     # volumes are defined.  We have no b/c considerations for implicit-root-fs
     # in the multi-volume case.
     rootfs_seen = False
+    # For UC20 a new gadget layout is used, in which only the seed partition
+    # needs to be explicitly created by ubuntu-image.  In ubuntu-image we will
+    # call this state as 'seeded'.
+    is_seeded = False
     farthest_offset = 0
     # This item is a dictionary so it can't possibly have duplicate keys.
     # That's okay because our StrictLoader above will already raise an
@@ -347,9 +358,15 @@ def parse(stream_or_string):
         bootloader_seen |= (bootloader is not None)
         image_id = image_spec.get('id')
         structures = []
+        structure_names = set()
         last_offset = 0
         for structure in image_spec['structure']:
             name = structure.get('name')
+            if name is not None:
+                if name in structure_names:
+                    raise GadgetSpecificationError(
+                        'Structure name "{}" is not unique'.format(name))
+                structure_names.add(name)
             offset = structure.get('offset')
             offset_write = structure.get('offset-write')
             size = structure['size']
@@ -482,6 +499,21 @@ def parse(stream_or_string):
                     raise GadgetSpecificationError(
                         '`role: system-data` structure must have an implicit '
                         "label, or 'writable': {}".format(filesystem_label))
+            elif structure_role is StructureRole.system_seed:
+                # The seed is good enough as a rootfs, snapd will create the
+                # writable partition on demand
+                rootfs_seen = True
+                # Also, since the gadget.yaml defines a system-seed partition,
+                # we can consider the image to be 'seeded'.  This basically
+                # changes the u-i build mechanism to only create the
+                # system-seed partition + all the the mbr/role-less partitions
+                # defined on the gadget.  All the others (system-boot,
+                # system-data etc.) will be created by snapd.
+                is_seeded = True
+                # Check if there is a filesystem label defined and, if not,
+                # use the implicit 'ubuntu-seed' label.
+                if filesystem_label is None:
+                    filesystem_label = 'ubuntu-seed'
             # The content will be one of two formats, and no mixing is
             # allowed.  I.e. even though multiple content sections are allowed
             # in a single structure, they must all be of type A or type B.  If
@@ -513,6 +545,10 @@ def parse(stream_or_string):
                 structure_type, structure_id, structure_role,
                 filesystem, filesystem_label,
                 content_specs))
+            # If we found a system-seed partition, stop looking at other
+            # parts.
+            if is_seeded:
+                break
         # Sort structures by their offset.
         volume_specs[image_name] = VolumeSpec(
             schema, bootloader, image_id, structures)
@@ -552,4 +588,4 @@ def parse(stream_or_string):
     if not bootloader_seen:
         raise GadgetSpecificationError('No bootloader structure named')
     return GadgetSpec(device_tree_origin, device_tree, volume_specs,
-                      defaults, format)
+                      defaults, format, is_seeded)
